@@ -389,6 +389,7 @@ extern struct litest_test_device litest_wacom_cintiq_24hdt_pad_device;
 extern struct litest_test_device litest_wacom_cintiq_13hdt_finger_device;
 extern struct litest_test_device litest_wacom_cintiq_13hdt_pen_device;
 extern struct litest_test_device litest_wacom_cintiq_13hdt_pad_device;
+extern struct litest_test_device litest_wacom_hid4800_tablet_device;
 
 struct litest_test_device* devices[] = {
 	&litest_synaptics_clickpad_device,
@@ -444,6 +445,7 @@ struct litest_test_device* devices[] = {
 	&litest_wacom_cintiq_13hdt_finger_device,
 	&litest_wacom_cintiq_13hdt_pen_device,
 	&litest_wacom_cintiq_13hdt_pad_device,
+	&litest_wacom_hid4800_tablet_device,
 	NULL,
 };
 
@@ -2873,6 +2875,12 @@ litest_timeout_gesture(void)
 	msleep(120);
 }
 
+void
+litest_timeout_trackpoint(void)
+{
+	msleep(320);
+}
+
 void
 litest_push_event_frame(struct litest_device *dev)
 {
