@@ -206,6 +206,7 @@ enum litest_device_type {
 	LITEST_WACOM_CINTIQ_13HDT_PEN,
 	LITEST_WACOM_CINTIQ_13HDT_PAD,
 	LITEST_WACOM_CINTIQ_13HDT_FINGER,
+	LITEST_WACOM_HID4800_PEN,
 };
 
 enum litest_device_feature {
@@ -631,6 +632,9 @@ litest_timeout_dwt_long(void);
 void
 litest_timeout_gesture(void);
 
+void
+litest_timeout_trackpoint(void);
+
 void
 litest_push_event_frame(struct litest_device *dev);
 
