@@ -658,6 +658,7 @@ START_TEST(touch_time_usec)
 	struct libinput *li = dev->libinput;
 	struct libinput_event *event;
 	struct libinput_event_touch *tev;
+	uint64_t time_usec;
 
 	litest_drain_events(dev->libinput);
 
@@ -667,12 +668,57 @@ START_TEST(touch_time_usec)
 
 	event = libinput_get_event(li);
 	tev = litest_is_touch_event(event, LIBINPUT_EVENT_TOUCH_DOWN);
+	time_usec = libinput_event_touch_get_time_usec(tev);
 	ck_assert_int_eq(libinput_event_touch_get_time(tev),
-			 libinput_event_touch_get_time_usec(tev) / 1000);
+			 (uint32_t) (time_usec / 1000));
 	libinput_event_destroy(event);
 }
 END_TEST
 
+START_TEST(touch_fuzz)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+	struct libinput_event *event;
+	int i;
+	int x = 700, y = 300;
+
+	litest_drain_events(dev->libinput);
+
+	litest_event(dev, EV_ABS, ABS_MT_TRACKING_ID, 30);
+	litest_event(dev, EV_ABS, ABS_MT_SLOT, 0);
+	litest_event(dev, EV_ABS, ABS_MT_POSITION_X, x);
+	litest_event(dev, EV_ABS, ABS_MT_POSITION_Y, y);
+	litest_event(dev, EV_KEY, BTN_TOUCH, 1);
+	litest_event(dev, EV_SYN, SYN_REPORT, 0);
+	libinput_dispatch(li);
+
+	event = libinput_get_event(li);
+	litest_is_touch_event(event, LIBINPUT_EVENT_TOUCH_DOWN);
+	libinput_event_destroy(event);
+	event = libinput_get_event(li);
+	litest_is_touch_event(event, LIBINPUT_EVENT_TOUCH_FRAME);
+	libinput_event_destroy(event);
+
+	litest_drain_events(li);
+
+	for (i = 0; i < 50; i++) {
+		if (i % 2) {
+			x++;
+			y--;
+		} else {
+			x--;
+			y++;
+		}
+		litest_event(dev, EV_ABS, ABS_MT_POSITION_X, x);
+		litest_event(dev, EV_ABS, ABS_MT_POSITION_Y, y);
+		litest_event(dev, EV_SYN, SYN_REPORT, 0);
+		libinput_dispatch(li);
+		litest_assert_empty_queue(li);
+	}
+}
+END_TEST
+
 void
 litest_setup_tests(void)
 {
@@ -701,4 +747,6 @@ litest_setup_tests(void)
 	litest_add_ranged("touch:state", touch_initial_state, LITEST_TOUCH, LITEST_PROTOCOL_A, &axes);
 
 	litest_add("touch:time", touch_time_usec, LITEST_TOUCH, LITEST_TOUCHPAD);
+
+	litest_add_for_device("touch:fuzz", touch_fuzz, LITEST_MULTITOUCH_FUZZ_SCREEN);
 }
